# This file is generated. Do not modify by hand.
from enum import Enum


class AxisType(Enum):
    """
    Denotes type of an axis and units it accepts.
    """

    UNKNOWN = 0
    LINEAR = 1
    ROTARY = 2
    PROCESS = 3
    LAMP = 4
