# This file is generated. Do not modify by hand.
from enum import Enum


class Errors(Enum):

    BAD_COMMAND = 0
    BAD_DATA = 1
    BINARY_COMMAND_FAILED = 2
    COMMAND_FAILED = 3
    COMMAND_PREEMPTED = 4
    COMMAND_TOO_LONG = 5
    CONNECTION_CLOSED = 6
    CONNECTION_FAILED = 7
    CONVERSION_FAILED = 8
    DEVICE_ADDRESS_CONFLICT = 9
    DEVICE_BUSY = 10
    DEVICE_DB_FAILED = 11
    DEVICE_DETECTION_FAILED = 12
    DEVICE_FAILED = 13
    DEVICE_NOT_IDENTIFIED = 14
    DRIVER_DISABLED = 15
    G_CODE_EXECUTION = 16
    G_CODE_SYNTAX = 17
    INCOMPATIBLE_SHARED_LIBRARY = 18
    INTERNAL_ERROR = 19
    INVALID_ARGUMENT = 20
    INVALID_DATA = 21
    INVALID_OPERATION = 22
    INVALID_PACKET = 23
    INVALID_PARK_STATE = 24
    INVALID_REQUEST_DATA = 25
    INVALID_RESPONSE = 26
    IO_CHANNEL_OUT_OF_RANGE = 27
    IO_FAILED = 28
    LOCKSTEP_ENABLED = 29
    LOCKSTEP_NOT_ENABLED = 30
    MOVEMENT_FAILED = 31
    MOVEMENT_INTERRUPTED = 32
    NO_DEVICE_FOUND = 33
    NO_VALUE_FOR_KEY = 34
    NOT_SUPPORTED = 35
    OPERATION_FAILED = 36
    OS_FAILED = 37
    OUT_OF_REQUEST_IDS = 38
    PVT_DISCONTINUITY = 39
    PVT_EXECUTION = 40
    PVT_MODE = 41
    PVT_MOVEMENT_FAILED = 42
    PVT_MOVEMENT_INTERRUPTED = 43
    PVT_SETUP_FAILED = 44
    REMOTE_MODE = 45
    REQUEST_TIMEOUT = 46
    SERIAL_PORT_BUSY = 47
    SET_DEVICE_STATE_FAILED = 48
    SET_PERIPHERAL_STATE_FAILED = 49
    SETTING_NOT_FOUND = 50
    STREAM_DISCONTINUITY = 51
    STREAM_EXECUTION = 52
    STREAM_MODE = 53
    STREAM_MOVEMENT_FAILED = 54
    STREAM_MOVEMENT_INTERRUPTED = 55
    STREAM_SETUP_FAILED = 56
    TIMEOUT = 57
    TRANSPORT_ALREADY_USED = 58
    UNKNOWN_REQUEST = 59
